#!/usr/bin/python

# script by Marcin Konicki <ahwayakchih@neoni.net>
# bootable code ported to python from http://stefanschramm.net/dev/makebootabletiny/makebootabletiny.c written by Stefan Schramm

import sys
import re
from subprocess import Popen,PIPE
import urllib2
import os
from struct import *

def targets():
	info = Popen(["fdisk", "-l", "-u"], stdout=PIPE).communicate()[0]
	result = {}
	for f in re.findall('(/dev/[^\s]+)\s+([a-z0-9\+\*]*)\s+(\d+)\s+(\d+)\s+(\d+\+?)\s+([a-z0-9\+\*]*)\s+([^\n]+)', info):
		result[f[0]] = f
	return result

def versions():
	return ('2', '2hybrid', '4', '4hybrid')

def usage():
	print "\nUsage: sudo haiku-nightly-installer GCCBUILD DEVICE"
	print "\nDownload and install latest Haiku nightly build (selected gcc build) to selected device and make it bootable."
	print "\nGCCBUILD can be one of:"
	print "\t2        - GCC2 build"
	print "\t2hybrid  - GCC2 build + GCC4 libraries"
	print "\t4        - GCC4 build"
	print "\t4hybrid  - GCC4 build + GCC2 libraries"
	print "\nHybrids try to make system compatible with both GCC versions.\nGCC2 versions should be able to run most of the software from BeBits.com.\nGCC4 versions are future, but Haiku R1 is plannned to be GCC2 mainly."
	print "\nDEVICE can be one of:"
	m = targets()
	for f in m:
		print "\t" + f + "\t\tcurrently is: " + m[f][6]
	print "\n\n"

def install_haiku(url, device):
	try:
		target = targets()[device]
	except:
		print "\nERROR: target device " + device + " not found."
		sys.exit(-1)

	# Download image
	print "\nDownloading nighlty image."
	Popen(["wget", "-c", url], stdout=PIPE).communicate()[0]
	if not os.path.exists(os.path.basename(url)):
		print "\nERROR: could not download " + url + " file."
		sys.exit(-1)

	# Unzip image
	print "\nUnzipping nighlty image."
	Popen(["unzip", "-o", os.path.basename(url)], stdout=PIPE).communicate()[0]
	if not os.path.exists('haiku-nightly.image'):
		print "\nERROR: could not unzip haiku-nightly.image file."
		sys.exit(-1)

	# Unmount target, just in case
	print "\nUnmounting " + device + "."
	Popen(["umount", device], stdout=PIPE).communicate()[0]

	# Write image to target
	print "\nWriting to " + device + "."
	Popen(["dd", "if=haiku-nightly.image", "of=" + device], stdout=PIPE).communicate()[0]

	# Make target bootable
	f = open(device, "r+")
	f.seek(506, os.SEEK_SET)
	code = f.read(4)
	code = str(unpack("L", code)[0])
	if code != target[2]:
		print "\nBoot position was set to: " + code + "."
		print "\nSetting boot position to: " + target[2] + "."
		f.seek(506, os.SEEK_SET)
		f.write(pack("L", long(target[2])))
	else:
		print "\nBoot position was already correct."
	f.close()
	print "\nDone."



if len(sys.argv) < 3 or sys.argv[1] in ('-h', '--help', 'help'):
	usage()
	sys.exit()

if sys.argv[1] not in versions():
	print "\nERROR: Wrong version name."
	print "\nUse --help to read instructions."
	print "\n"
	sys.exit()

if sys.argv[2] not in targets():
	print "\nERROR: Wrong device selected."
	print "\nUse --help to read instructions."
	print "\n"
	sys.exit()


f = urllib2.urlopen('http://haiku-files.org/raw/')
html = f.read()
f.close()
m = re.findall('\<a href="(http://haiku-files.org/raw/haiku-nightly-(r[\d]+)-x86gcc(\d(hybrid)?)-raw\.zip)"[^\>]*\>\s*([\d\.]+(MB|GB))\s*\</a\>', html)

if len(m) < 1:
	print "\nERROR: Could not find recent nightly raw image."
	print "\n"
	sys.exit()

revision = m[0][1]
for f in m:
	if f[1] == revision and f[2] == sys.argv[1]:
		install_haiku(f[0], sys.argv[2])
		sys.exit()
